package com.ub.alumnes.mmorenfe.pfg_app;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.app.Activity;
import android.app.ListFragment;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.View;
import android.widget.ListView;

import com.ub.alumnes.mmorenfe.pfg_app.adapters.TestAdapter;
import com.ub.alumnes.mmorenfe.pfg_app.model.Pregunta;
import com.ub.alumnes.mmorenfe.pfg_app.model.Resposta;
import com.ub.alumnes.mmorenfe.pfg_app.model.Test;

public class AvaluacioPerTestListFragment extends ListFragment {

	private List<Test> tests;
	protected static Map<Integer, Test> testMap;
	
	private TestAdapter ta;
	
	private int currentPosition;

	/**
	 * The serialization (saved instance state) Bundle key representing the
	 * activated item position. Only used on tablets.
	 */
	private static final String STATE_ACTIVATED_POSITION = "activated_position";

	/**
	 * The fragment's current callback object, which is notified of list item
	 * clicks.
	 */
	private Callbacks mCallbacks = sDummyCallbacks;

	/**
	 * The current activated item position. Only used on tablets.
	 */
	private int mActivatedPosition = ListView.INVALID_POSITION;

	/**
	 * A callback interface that all activities containing this fragment must
	 * implement. This mechanism allows activities to be notified of item
	 * selections.
	 */
	public interface Callbacks {
		/**
		 * Callback for when an item has been selected.
		 */
		public void onItemSelected(int id, int pos);
	}

	/**
	 * A dummy implementation of the {@link Callbacks} interface that does
	 * nothing. Used only when this fragment is not attached to an activity.
	 */
	private static Callbacks sDummyCallbacks = new Callbacks() {
		@Override
		public void onItemSelected(int id, int pos) {
		}
	};

	/**
	 * Mandatory empty constructor for the fragment manager to instantiate the
	 * fragment (e.g. upon screen orientation changes).
	 */
	public AvaluacioPerTestListFragment() {
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		SharedPreferences prefs = getActivity().getSharedPreferences("account",
				Context.MODE_PRIVATE);

		ta =new TestAdapter(getActivity(),
				R.layout.test_row_element, downloadTests(), prefs.getBoolean("isProfessor", false));
		setListAdapter(ta);
	}

	@Override
	public void onViewCreated(View view, Bundle savedInstanceState) {
		super.onViewCreated(view, savedInstanceState);

		// Restore the previously serialized activated item position.
		if (savedInstanceState != null
				&& savedInstanceState.containsKey(STATE_ACTIVATED_POSITION)) {
			setActivatedPosition(savedInstanceState
					.getInt(STATE_ACTIVATED_POSITION));
		}
	}
	
	

	@Override
	public void onStart() {
		super.onStart();
		ta.setNotifyOnChange(true);
		setActivateOnItemClick(true);
		
		getListView().setSelector(getResources().getDrawable(R.drawable.list_item_selector));
	}
	
	

	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);

		testMap = new HashMap<Integer, Test>();

		// Activities containing this fragment must implement its callbacks.
		if (!(activity instanceof Callbacks)) {
			throw new IllegalStateException(
					"Activity must implement fragment's callbacks.");
		}

		mCallbacks = (Callbacks) activity;
	}

	@Override
	public void onDetach() {
		super.onDetach();

		// Reset the active callbacks interface to the dummy implementation.
		mCallbacks = sDummyCallbacks;
	}

	@Override
	public void onListItemClick(ListView listView, View view, int position,
			long id) {
		super.onListItemClick(listView, view, position, id);

		currentPosition = position;
		// Notify the active callbacks interface (the activity, if the
		// fragment is attached to one) that an item has been selected.
		mCallbacks.onItemSelected(tests.get(position).getId(), position);
	}

	@Override
	public void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		if (mActivatedPosition != ListView.INVALID_POSITION) {
			// Serialize and persist the activated item position.
			outState.putInt(STATE_ACTIVATED_POSITION, mActivatedPosition);
		}
	}

	/**
	 * Turns on activate-on-click mode. When this mode is on, list items will be
	 * given the 'activated' state when touched.
	 */
	public void setActivateOnItemClick(boolean activateOnItemClick) {
		// When setting CHOICE_MODE_SINGLE, ListView will automatically
		// give items the 'activated' state when touched.
		getListView().setChoiceMode(
				activateOnItemClick ? ListView.CHOICE_MODE_SINGLE
						: ListView.CHOICE_MODE_NONE);
	}

	private void setActivatedPosition(int position) {
		if (position == ListView.INVALID_POSITION) {
			getListView().setItemChecked(mActivatedPosition, false);
		} else {
			getListView().setItemChecked(position, true);
		}

		mActivatedPosition = position;
	}

	private List<Test> downloadTests() {
		// per el metode de descarrega.
		tests = new ArrayList<Test>();
		tests = generateHardcodedList();

		for (Test test : tests) {
			testMap.put(test.getId(), test);
		}

		return tests;
	}


	private List<Test> generateHardcodedList() {

		// tests
		Test t1;
		Test t2;

		// test list
		ArrayList<Test> tests = new ArrayList<Test>();

		// respostes per la pregunta 1 del tests 1
		Resposta r1p1t1 = new Resposta(1, "Resposta 1", false);
		Resposta r2p1t1 = new Resposta(2, "Resposta 2", false);
		Resposta r3p1t1 = new Resposta(3, "Resposta 3", true);
		Resposta r4p1t1 = new Resposta(4, "Resposta 4", false);

		List<Resposta> respostes = new ArrayList<Resposta>(4);

		respostes.add(r1p1t1);
		respostes.add(r2p1t1);
		respostes.add(r3p1t1);
		respostes.add(r4p1t1);

		// pregunta 1 del test 1
		Pregunta p1t1 = new Pregunta(1, "pregunta 1", respostes);

		// respostes pre la pregunta 2 del test 1
		Resposta r1p2t1 = new Resposta(5, "Resposta 1", false);
		Resposta r2p2t1 = new Resposta(6, "Resposta 2", false);
		Resposta r3p2t1 = new Resposta(7, "Resposta 3", false);
		Resposta r4p2t1 = new Resposta(8, "Resposta 4", true);

		List<Resposta> respostes2 = new ArrayList<Resposta>(4);

		respostes2.add(r1p2t1);
		respostes2.add(r2p2t1);
		respostes2.add(r3p2t1);
		respostes2.add(r4p2t1);

		Pregunta p2t1 = new Pregunta(2, "pregunta 2", respostes2);

		List<Pregunta> preguntes1 = new ArrayList<Pregunta>();

		preguntes1.add(p1t1);
		preguntes1.add(p2t1);

		// respostes per la pregunta 1 del test 2
		Resposta r1p1t2 = new Resposta(9, "Resposta 1", true);
		Resposta r2p1t2 = new Resposta(10, "Resposta 2", false);
		Resposta r3p1t2 = new Resposta(11, "Resposta 3", false);
		Resposta r4p1t2 = new Resposta(12, "Resposta 4", false);

		List<Resposta> respostes3 = new ArrayList<Resposta>(4);

		respostes3.add(r1p1t2);
		respostes3.add(r2p1t2);
		respostes3.add(r3p1t2);
		respostes3.add(r4p1t2);

		Pregunta p1t2 = new Pregunta(3, "pregunta 1", respostes3);

		// respostes per la pregunta 2 del test 2
		Resposta r1p2t2 = new Resposta(13, "Resposta 1", false);
		Resposta r2p2t2 = new Resposta(14, "Resposta 2", false);
		Resposta r3p2t2 = new Resposta(15, "Resposta 3", true);
		Resposta r4p2t2 = new Resposta(16, "Resposta 4", false);

		List<Resposta> respostes4 = new ArrayList<Resposta>(4);

		respostes4.add(r1p2t2);
		respostes4.add(r2p2t2);
		respostes4.add(r3p2t2);
		respostes4.add(r4p2t2);

		Pregunta p2t2 = new Pregunta(4, "pregunta 2", respostes4);

		List<Pregunta> preguntes2 = new ArrayList<Pregunta>();

		preguntes2.add(p1t2);
		preguntes2.add(p2t2);

		t1 = new Test(1, "test 1", preguntes1, true);
		t2 = new Test(2, "test 2", preguntes2, false);

		tests.add(t1);
		tests.add(t2);

		return tests;
	}
	
	public void addTest(final Test test) {
		tests.add(test);
		if (testMap.containsKey(test.getId())) {
			testMap.remove(test.getId());
		}
		ta.notifyDataSetChanged();
		
	}
	
	public void setTest (final Test test) {
		
		for (int i = 0; i < tests.size(); i++) {
			if (tests.get(i).getId() == test.getId()) {
				tests.set(i, test);
			}
		}
		
		ta.notifyDataSetChanged();
	}
}
