package com.ub.alumnes.mmorenfe.pfg_app;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.TargetApi;
import android.app.Activity;
import android.app.DialogFragment;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.ub.alumnes.mmorenfe.pfg_app.model.Pregunta;
import com.ub.alumnes.mmorenfe.pfg_app.model.Resposta;
import com.ub.alumnes.mmorenfe.pfg_app.model.Test;

public class FragmentCreaPregunta extends DialogFragment {

	View containerDialog;

	private EditText title;
	private EditText resposta1;
	private EditText resposta2;
	private EditText resposta3;
	private EditText resposta4;

	private Button send;
	
	private Test test;

	private DoCreatePregunta task = null;

	private View status;
	private View content;

	private TextView statusText;

	private Pregunta pregunta;

	public interface OnCreatePregunta {

		public void onPreguntaCreated(final Test test);
	}

	/**
	 * A dummy implementation of the {@link OnCreatePregunta} interface that
	 * does nothing. Used only when this fragment is not attached to an
	 * activity.
	 */
	private static OnCreatePregunta cDummy = new OnCreatePregunta() {

		@Override
		public void onPreguntaCreated(final Test test) {

		}
	};

	public OnCreatePregunta cCallback = cDummy;

	public FragmentCreaPregunta(final Test test) {
		this.test = test;
	}
	
	public FragmentCreaPregunta() {
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		containerDialog = inflater.inflate(R.layout.layout_create_pregunta,
				container, false);

		return containerDialog;
	}

	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		try {
			cCallback = (OnCreatePregunta) activity;
		} catch (ClassCastException e) {
			throw new ClassCastException(activity.toString()
					+ " must implement OnCreatePregunta");
		}
	}

	@Override
	public void onDetach() {
		super.onDetach();
		cCallback = cDummy;
	}

	@Override
	public void onStart() {
		super.onStart();

		// get ui objects
		title = (EditText) containerDialog
				.findViewById(R.id.editTextTitleQuestion);

		resposta1 = (EditText) containerDialog
				.findViewById(R.id.editTextAnswer1);
		resposta2 = (EditText) containerDialog
				.findViewById(R.id.editTextAnswer2);
		resposta3 = (EditText) containerDialog
				.findViewById(R.id.editTextAnswer3);
		resposta4 = (EditText) containerDialog
				.findViewById(R.id.editTextAnswer4);

		send = (Button) containerDialog.findViewById(R.id.buttonFinishQuestion);

		status = containerDialog.findViewById(R.id.pregunta_status);
		content = containerDialog.findViewById(R.id.pregunta_content);
		
		statusText = (TextView) containerDialog
				.findViewById(R.id.pregunta_status_message);
		
		send.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				validate();
			}
		});

	}

	private void validate() {
		if (task != null) {
			return;
		}

		// reset errors
		title.setError(null);
		resposta1.setError(null);
		resposta2.setError(null);
		resposta3.setError(null);
		resposta4.setError(null);

		boolean cancel = false;
		View focusView = null;

		if (TextUtils.isEmpty(resposta4.getText().toString())) {
			resposta4.setError(getString(R.string.error_empty));
			cancel = true;
			focusView = resposta4;
		}

		if (TextUtils.isEmpty(resposta3.getText().toString())) {
			resposta3.setError(getString(R.string.error_empty));
			cancel = true;
			focusView = resposta3;
		}

		if (TextUtils.isEmpty(resposta2.getText().toString())) {
			resposta2.setError(getString(R.string.error_empty));
			cancel = true;
			focusView = resposta2;
		}

		if (TextUtils.isEmpty(resposta1.getText().toString())) {
			resposta1.setError(getString(R.string.error_empty));
			cancel = true;
			focusView = resposta1;
		}

		if (cancel) {
			// There was an error; don't attempt login and focus the first
			// form field with an error.
			focusView.requestFocus();
		} else {
			// Show a progress spinner, and kick off a background task to
			// perform the user login attempt.
			statusText.setText(R.string.login_progress_signing_in);
			showProgress(true);
			task = new DoCreatePregunta();
			task.execute((Void) null);
			try {
				showProgress(true);
				test.getPreguntes().add(task.get());
				cCallback.onPreguntaCreated(test);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (ExecutionException e) {
				e.printStackTrace();
			}
		}

	}

	/**
	 * Aquesta clase fara la petici'o al servidor per crear una pregunta.
	 * 
	 * @author mmorenfe
	 * 
	 */
	private class DoCreatePregunta extends AsyncTask<Void, Void, Pregunta> {

		@Override
		protected Pregunta doInBackground(Void... params) {

			// TODO: petici'o al servidor per crear una pregunta i rebre la
			// pregunta creada amb la id corresponent
			try {
				// Simulate network access.
				Thread.sleep(2000);
			} catch (InterruptedException e) {
				return null;
			}

			Resposta r1 = new Resposta(0, resposta1.getText().toString(), false);
			Resposta r2 = new Resposta(0, resposta2.getText().toString(), false);
			Resposta r3 = new Resposta(0, resposta3.getText().toString(), false);
			Resposta r4 = new Resposta(0, resposta4.getText().toString(), false);

			List<Resposta> respostes = new ArrayList<Resposta>();

			respostes.add(r1);
			respostes.add(r2);
			respostes.add(r3);
			respostes.add(r4);

			pregunta = new Pregunta((int) (Math.random() * 999), title
					.getText().toString(), respostes);

			return pregunta;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Pregunta result) {
			task = null;
			showProgress(false);
			dismiss();

		}

	}

	/**
	 * Shows the progress UI and hides the login form.
	 */
	@TargetApi(Build.VERSION_CODES.HONEYCOMB_MR2)
	private void showProgress(final boolean show) {
		// On Honeycomb MR2 we have the ViewPropertyAnimator APIs, which allow
		// for very easy animations. If available, use these APIs to fade-in
		// the progress spinner.
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR2) {
			int shortAnimTime = getResources().getInteger(
					android.R.integer.config_shortAnimTime);

			status.setVisibility(View.VISIBLE);
			status.animate().setDuration(shortAnimTime).alpha(show ? 1 : 0)
					.setListener(new AnimatorListenerAdapter() {
						@Override
						public void onAnimationEnd(Animator animation) {
							status.setVisibility(show ? View.VISIBLE
									: View.GONE);
						}
					});

			content.setVisibility(View.VISIBLE);
			content.animate().setDuration(shortAnimTime).alpha(show ? 0 : 1)
					.setListener(new AnimatorListenerAdapter() {
						@Override
						public void onAnimationEnd(Animator animation) {
							content.setVisibility(show ? View.GONE
									: View.VISIBLE);
						}
					});
		} else {
			// The ViewPropertyAnimator APIs are not available, so simply show
			// and hide the relevant UI components.
			status.setVisibility(show ? View.VISIBLE : View.GONE);
			content.setVisibility(show ? View.GONE : View.VISIBLE);
		}
	}

}
