package com.ub.alumnes.mmorenfe.pfg_app;

import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.app.DialogFragment;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ListView;

import com.ub.alumnes.mmorenfe.pfg_app.adapters.PreguntaAdapter;
import com.ub.alumnes.mmorenfe.pfg_app.model.Pregunta;
import com.ub.alumnes.mmorenfe.pfg_app.model.Resposta;
import com.ub.alumnes.mmorenfe.pfg_app.model.Test;

public class FragmentCreateEditTest extends DialogFragment {

	private static final int MODE_CREATE = 0;
	private static final int MODE_EDIT = 1;

	private int mode;
	private Test test;


	private EditText editTitle;
	private ListView listPreguntes;
	private ImageButton btnAddPregunta;
	private Button btnSend;
	private PreguntaAdapter pa;

	View containerDialog;

	/**
	 * Constructor. we have to pass a test with id == -1 to let app know that we
	 * try to create a new test.
	 * 
	 * @param test
	 *            the tests to edit
	 */
	public FragmentCreateEditTest(final Test test, final int mode) {
		this.mode = mode;
		this.test = test;
	}

	private OnCreateTest createCallback = cDummy;

	private OnEditTest editCallback = eDummy;
	
	private OnCreatePreguntaRequest preguntaCallbak = pDummy;

	/**
	 * A callback interface that all activities containing this fragment must
	 * implement. This mechanism allows activities to be notified of Test
	 * creation
	 */
	public interface OnCreateTest {
		/**
		 * Callback for when an item has been selected.
		 */
		public void onTestCreated(final Test test);
	}

	/**
	 * A callback interface that all activities containing this fragment must
	 * implement. This mechanism allows activities to be notified of test
	 * edition
	 */
	public interface OnEditTest {
		/**
		 * Callback for when an item has been selected.
		 */
		public void onTestEdited(final Test test);
	}
	
	/**
	 * A callback interface that all activities containing this fragment must
	 * implement. This mechanism allows activities to be notified of request to
	 * create a question.
	 */
	public interface OnCreatePreguntaRequest {
		
		public void onRequest(final Test test);
	}
	
	
	/**
	 * A dummy implementation of the {@link OnCreatePreguntaRequest} interface that does
	 * nothing. Used only when this fragment is not attached to an activity.
	 */
	private static OnCreatePreguntaRequest pDummy = new OnCreatePreguntaRequest() {
		
		@Override
		public void onRequest(final Test test) {
			
		}
	};

	/**
	 * A dummy implementation of the {@link OnCreateTest} interface that does
	 * nothing. Used only when this fragment is not attached to an activity.
	 */
	private static OnCreateTest cDummy = new OnCreateTest() {

		@Override
		public void onTestCreated(Test test) {

		}
	};

	/**
	 * A dummy implementation of the {@link OnEditTest} interface that does
	 * nothing. Used only when this fragment is not attached to an activity.
	 */
	private static OnEditTest eDummy = new OnEditTest() {

		@Override
		public void onTestEdited(Test test) {

		}
	};

	/**
	 * Default constructor with no fields.
	 * 
	 */
	public FragmentCreateEditTest() {
		super();
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		containerDialog = inflater.inflate(R.layout.dialog_test, container,
				false);

		return containerDialog;
	}

	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);

		try {
			createCallback = (OnCreateTest) activity;
		} catch (ClassCastException e) {
			throw new ClassCastException(activity.toString()
					+ " must implement OnCreateTest");
		}

		try {
			editCallback = (OnEditTest) activity;
		} catch (ClassCastException e) {
			throw new ClassCastException(activity.toString()
					+ " must implement OnEditTest");
		}
		
		try {
			preguntaCallbak = (OnCreatePreguntaRequest) activity;
		} catch (ClassCastException e) {
			throw new ClassCastException(activity.toString()
					+ " must implement OnCreatePreguntaRequest");
		}
	}

	@Override
	public void onStart() {
		super.onStart();
		SharedPreferences prefs = getActivity().getSharedPreferences("account",
				Context.MODE_PRIVATE);
		editTitle = (EditText) containerDialog.findViewById(R.id.editTitle);
		listPreguntes = (ListView) containerDialog
				.findViewById(R.id.question_list);
		btnAddPregunta = (ImageButton) containerDialog
				.findViewById(R.id.btn_add_question);
		btnSend = (Button) containerDialog.findViewById(R.id.button_create);
		
		btnSend.setOnClickListener(new Button.OnClickListener() {

			@Override
			public void onClick(View v) {
				if (mode == MODE_CREATE) {
					test.setId((int) (Math.random() * 10000));
					createCallback.onTestCreated(test);
				} else {
					test.setTitol(editTitle.getText().toString());
					editCallback.onTestEdited(test);
				}
			}
			
		});
		
		if (mode == MODE_CREATE) {
			
			test.setPreguntes(new ArrayList<Pregunta>());
		}
		
		
			pa = new PreguntaAdapter(getActivity(), R.layout.pregunta_test,
					test.getPreguntes(), prefs.getBoolean("isProfessor", false), mode);
		
		listPreguntes.setAdapter(pa);

		btnAddPregunta.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				preguntaCallbak.onRequest(test);
			}
		});

		switch (mode) {
		case MODE_CREATE:
			break;
		case MODE_EDIT:
			editTitle.setText(test.getTitol());
			break;
		default:
			break;
		}

	}

	@Override
	public void onDetach() {
		super.onDetach();

		// Reset the active callbacks interface to the dummy implementation.
		createCallback = cDummy;

		editCallback = eDummy;
	}


}
