package com.ub.alumnes.mmorenfe.pfg_app.utils;

import android.content.Context;
import android.os.AsyncTask;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.*;
import android.widget.LinearLayout.LayoutParams;
import com.ub.alumnes.mmorenfe.pfg_app.R;
import com.ub.alumnes.mmorenfe.pfg_app.model.Pregunta;
import com.ub.alumnes.mmorenfe.pfg_app.model.Resposta;

import java.util.List;
import java.util.concurrent.ExecutionException;

public class TestUtils {

    private Context context;

    public TestUtils(final Context context) {
        this.context = context;
    }

    public void generateQuestion(final Pregunta pregunta, final RadioGroup rg,
                                 final LinearLayout ll) {
        for (Resposta r : pregunta.getRespostes()) {
            RadioButton rb = new RadioButton(context);
            rb.setLayoutParams(new LinearLayout.LayoutParams(
                    LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
            rb.setId(r.getId());
            rb.setGravity(Gravity.CENTER);
            rb.setText(r.getText());

            if (r.isCorrecta()) {
                rb.setChecked(true);
            } else {
                rb.setChecked(false);
            }

            rg.addView(rb);

            EditText et = new EditText(context);
            et.setLayoutParams(new LinearLayout.LayoutParams(
                    LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
            et.setId(r.getId());
            et.setGravity(Gravity.CENTER);
            et.setHint(r.getText());

            et.setOnClickListener(new View.OnClickListener() {

                @Override
                public void onClick(View v) {
                    v.requestFocus();
                }
            });

            ll.addView(et);
        }
    }

    public void addResposta(Pregunta pregunta, final Resposta r, RadioGroup rg, LinearLayout ll) {

        DoCreateAnswer task = new DoCreateAnswer();
        task.execute(r);

        Resposta resposta = null;
        try {
            resposta = task.get();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }

        assert resposta.getId() >= 0;

        String hint = context.getString(R.string.novaResposta);

        pregunta.getRespostes().add(resposta);

        RadioButton rb = new RadioButton(context);
        rb.setLayoutParams(new LinearLayout.LayoutParams(
                LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));

        rb.setId(0);
        rb.setGravity(Gravity.CENTER);

        rg.addView(rb);

        EditText et = new EditText(context);
        et.setLayoutParams(new LinearLayout.LayoutParams(
                LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
        et.setId(0);
        et.setGravity(Gravity.CENTER);

        et.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View vw) {
                vw.requestFocus();
            }
        });

        ll.addView(et);
    }

    public void actualitzaPregunta(final Pregunta pregunta, RadioGroup rg, LinearLayout ll) {
        List<Resposta> respostes = pregunta.getRespostes();
        for (int i = 0; i < respostes.size(); i++) {
            EditText ed = (EditText) ll.findViewById(respostes.get(i).getId());
            RadioButton rb = (RadioButton) rg.findViewById(respostes.get(i).getId());

            if (TextUtils.isEmpty(ed.getText())) {
                respostes.remove(i);
                ll.removeView(ed);
                rg.removeView(rb);
            } else {
                ed.setHint(ed.getText().toString());
                rb.setText(ed.getText().toString());
                respostes.get(i).setText(ed.getText().toString());
            }

        }
    }

    public RelativeLayout generaLayoutPregunta(final boolean isProfessor, final Pregunta pregunta) {

        RelativeLayout v;
        LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        //carreguem la plantilla en aquest layout.
        v = (RelativeLayout) inflater.inflate(R.layout.pregunta_test, null);

        final RelativeLayout edita = (RelativeLayout) v
                .findViewById(R.id.layout_edita_pregunta);
        final LinearLayout respon = (LinearLayout) v
                .findViewById(R.id.layout_respon_pregunta);

        ImageButton btnEdit = (ImageButton) v
                .findViewById(R.id.btnEdit);
        ImageButton btnSend = (ImageButton) edita
                .findViewById(R.id.btnSend);

        final RadioButton rb1 = (RadioButton) v.findViewById(R.id.r1);
        final RadioButton rb2 = (RadioButton) v.findViewById(R.id.r2);
        final RadioButton rb3 = (RadioButton) v.findViewById(R.id.r3);
        final RadioButton rb4 = (RadioButton) v.findViewById(R.id.r4);


        final EditText t1 = (EditText) v.findViewById(R.id.t1);
        final EditText t2 = (EditText) v.findViewById(R.id.t2);
        final EditText t3 = (EditText) v.findViewById(R.id.t3);
        final EditText t4 = (EditText) v.findViewById(R.id.t4);

        final EditText editTitle = (EditText) v.findViewById(R.id.tTitol);
        final TextView title = (TextView) v.findViewById(R.id.textQuestionTitle);

        RadioGroup rg = (RadioGroup) v.findViewById(R.id.radioGroupAnswers);
        LinearLayout ll = (LinearLayout) v
                .findViewById(R.id.layoutPreguntesEditades);


        if (!isProfessor) {
            btnEdit.setVisibility(View.INVISIBLE);
        }

        if (pregunta != null) {
            rb1.setText(pregunta.getRespostes().get(0).getText());
            rb2.setText(pregunta.getRespostes().get(1).getText());
            rb3.setText(pregunta.getRespostes().get(2).getText());
            rb4.setText(pregunta.getRespostes().get(3).getText());

            t1.setText(pregunta.getRespostes().get(0).getText());
            t2.setText(pregunta.getRespostes().get(1).getText());
            t3.setText(pregunta.getRespostes().get(2).getText());
            t4.setText(pregunta.getRespostes().get(3).getText());

            editTitle.setText(pregunta.getText());
            title.setText(pregunta.getText());

        }

        btnEdit.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                respon.setVisibility(View.GONE);
                edita.setVisibility(View.VISIBLE);
            }
        });

        btnSend.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {

                t1.setError(null);
                t2.setError(null);
                t3.setError(null);
                t4.setError(null);

                boolean cancel = false;
                View focusView = null;

                if (TextUtils.isEmpty(t4.getText())) {
                    t4.setError(context.getString(R.string.error_empty));
                    cancel = true;
                    focusView = t4;
                }

                if (TextUtils.isEmpty(t3.getText())) {
                    t3.setError(context.getString(R.string.error_empty));
                    cancel = true;
                    focusView = t3;
                }

                if (TextUtils.isEmpty(t2.getText())) {
                    t2.setError(context.getString(R.string.error_empty));
                    cancel = true;
                    focusView = t2;
                }

                if (TextUtils.isEmpty(t1.getText())) {
                    t1.setError(context.getString(R.string.error_empty));
                    cancel = true;
                    focusView = t1;
                }

                if (TextUtils.isEmpty(editTitle.getText())) {
                    editTitle.setError(context.getString(R.string.error_empty));
                    cancel = true;
                    focusView = editTitle;
                }

                if (cancel) {
                    focusView.requestFocus();
                } else {
                    rb1.setText(t1.getText().toString());
                    rb2.setText(t2.getText().toString());
                    rb3.setText(t3.getText().toString());
                    rb4.setText(t4.getText().toString());

                    title.setText(editTitle.getText().toString());

                    respon.setVisibility(View.VISIBLE);
                    edita.setVisibility(View.GONE);
                }

            }
        });

        return v;
    }

    private class DoCreateAnswer extends AsyncTask<Resposta, Void, Resposta> {

        @Override
        protected Resposta doInBackground(Resposta... params) {


            Resposta r = new Resposta((int)(Math.random()*95),context.getString(R.string.novaResposta),false);
            
            return r;
        }

        @Override
        protected void onPostExecute(Resposta result) {
            super.onPostExecute(result);


        }

    }
}
